System.register(["./RangeDecoder", "./OutWindow", "./BitTreeDecoder", "./LenDecoder", "./LZMA", "../utils/MEMORY"], function(exports_1, context_1) {
    "use strict";
    var __moduleName = context_1 && context_1.id;
    var RangeDecoder_1, OutWindow_1, BitTreeDecoder_1, LenDecoder_1, LZMA_1, MEMORY_1;
    var LzmaDecoder;
    return {
        setters:[
            function (RangeDecoder_1_1) {
                RangeDecoder_1 = RangeDecoder_1_1;
            },
            function (OutWindow_1_1) {
                OutWindow_1 = OutWindow_1_1;
            },
            function (BitTreeDecoder_1_1) {
                BitTreeDecoder_1 = BitTreeDecoder_1_1;
            },
            function (LenDecoder_1_1) {
                LenDecoder_1 = LenDecoder_1_1;
            },
            function (LZMA_1_1) {
                LZMA_1 = LZMA_1_1;
            },
            function (MEMORY_1_1) {
                MEMORY_1 = MEMORY_1_1;
            }],
        execute: function() {
            LzmaDecoder = (function () {
                function LzmaDecoder() {
                    this.posSlotDecoder = BitTreeDecoder_1.BitTreeDecoder.constructArray(6, LZMA_1.LZMA.kNumLenToPosStates);
                    this.alignDecoder = new BitTreeDecoder_1.BitTreeDecoder(LZMA_1.LZMA.kNumAlignBits);
                    this.posDecoders = new Uint16Array(1 + LZMA_1.LZMA.kNumFullDistances - LZMA_1.LZMA.kEndPosModelIndex);
                    this.isMatch = new Uint16Array(LZMA_1.LZMA.kNumStates << LZMA_1.LZMA.kNumPosBitsMax);
                    this.isRep = new Uint16Array(LZMA_1.LZMA.kNumStates);
                    this.isRepG0 = new Uint16Array(LZMA_1.LZMA.kNumStates);
                    this.isRepG1 = new Uint16Array(LZMA_1.LZMA.kNumStates);
                    this.isRepG2 = new Uint16Array(LZMA_1.LZMA.kNumStates);
                    this.isRep0Long = new Uint16Array(LZMA_1.LZMA.kNumStates << LZMA_1.LZMA.kNumPosBitsMax);
                    this.lenDecoder = new LenDecoder_1.LenDecoder();
                    this.repLenDecoder = new LenDecoder_1.LenDecoder();
                    this.rangeDec = new RangeDecoder_1.RangeDecoder();
                    this.outWindow = new OutWindow_1.OutWindow();
                }
                LzmaDecoder.prototype.init = function () {
                    this.loc1 = MEMORY_1.MEMORY.getUint32() | 0;
                    this.loc2 = MEMORY_1.MEMORY.getUint32() | 0;
                    this.matchBitI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.matchByteI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.bitI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.symbolI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.prevByteI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.litStateI = MEMORY_1.MEMORY.getUint16() | 0;
                    this.initLiterals();
                    this.initDist();
                    LZMA_1.LZMA.INIT_PROBS(this.isMatch);
                    LZMA_1.LZMA.INIT_PROBS(this.isRep);
                    LZMA_1.LZMA.INIT_PROBS(this.isRepG0);
                    LZMA_1.LZMA.INIT_PROBS(this.isRepG1);
                    LZMA_1.LZMA.INIT_PROBS(this.isRepG2);
                    LZMA_1.LZMA.INIT_PROBS(this.isRep0Long);
                    this.lenDecoder.init();
                    this.repLenDecoder.init();
                };
                LzmaDecoder.prototype.create = function () {
                    this.outWindow.create(this.dictSize);
                    this.createLiterals();
                };
                LzmaDecoder.prototype.createLiterals = function () {
                    this.litProbs = new Uint16Array(0x300 << (this.lc + this.lp));
                };
                LzmaDecoder.prototype.initLiterals = function () {
                    var num = 0x300 << (this.lc + this.lp);
                    for (var i = 0; i < num; i++) {
                        this.litProbs[i] = LZMA_1.LZMA.PROB_INIT_VAL;
                    }
                };
                LzmaDecoder.prototype.decodeLiteral = function (state, rep0) {
                    MEMORY_1.MEMORY.u16[this.prevByteI] = 0;
                    if (!this.outWindow.isEmpty())
                        MEMORY_1.MEMORY.u16[this.prevByteI] = this.outWindow.getByte(1);
                    MEMORY_1.MEMORY.u16[this.symbolI] = 1;
                    MEMORY_1.MEMORY.u16[this.litStateI] = ((this.outWindow.totalPos & ((1 << this.lp) - 1)) << this.lc) + (MEMORY_1.MEMORY.u16[this.prevByteI] >>> (8 - this.lc));
                    var probsOffset = (0x300 * MEMORY_1.MEMORY.u16[this.litStateI]) | 0;
                    if (state >= 7) {
                        MEMORY_1.MEMORY.u16[this.matchByteI] = this.outWindow.getByte(rep0 + 1);
                        do {
                            MEMORY_1.MEMORY.u16[this.matchBitI] = (MEMORY_1.MEMORY.u16[this.matchByteI] >>> 7) & 1;
                            MEMORY_1.MEMORY.u16[this.matchByteI] <<= 1;
                            MEMORY_1.MEMORY.u16[this.bitI] = this.rangeDec.decodeBit(this.litProbs, probsOffset + ((1 + MEMORY_1.MEMORY.u16[this.matchBitI]) << 8) + MEMORY_1.MEMORY.u16[this.symbolI]);
                            MEMORY_1.MEMORY.u16[this.symbolI] = (MEMORY_1.MEMORY.u16[this.symbolI] << 1) | MEMORY_1.MEMORY.u16[this.bitI];
                            if (MEMORY_1.MEMORY.u16[this.matchBitI] != MEMORY_1.MEMORY.u16[this.bitI])
                                break;
                        } while (MEMORY_1.MEMORY.u16[this.symbolI] < 0x100);
                    }
                    while (MEMORY_1.MEMORY.u16[this.symbolI] < 0x100) {
                        MEMORY_1.MEMORY.u16[this.symbolI] = (MEMORY_1.MEMORY.u16[this.symbolI] << 1) | this.rangeDec.decodeBit(this.litProbs, probsOffset + MEMORY_1.MEMORY.u16[this.symbolI]);
                    }
                    this.outWindow.putByte(MEMORY_1.MEMORY.u16[this.symbolI] - 0x100);
                };
                LzmaDecoder.prototype.decodeDistance = function (len) {
                    var lenState = len;
                    if (lenState > LZMA_1.LZMA.kNumLenToPosStates - 1)
                        lenState = LZMA_1.LZMA.kNumLenToPosStates - 1;
                    var posSlot = this.posSlotDecoder[lenState].decode(this.rangeDec);
                    if (posSlot < 4)
                        return posSlot;
                    var numDirectBits = ((posSlot >>> 1) - 1);
                    MEMORY_1.MEMORY.u32[this.loc1] = ((2 | (posSlot & 1)) << numDirectBits);
                    if (posSlot < LZMA_1.LZMA.kEndPosModelIndex) {
                        MEMORY_1.MEMORY.u32[this.loc1] += LZMA_1.LZMA.BitTreeReverseDecode(this.posDecoders, numDirectBits, this.rangeDec, MEMORY_1.MEMORY.u32[this.loc1] - posSlot);
                    }
                    else {
                        MEMORY_1.MEMORY.u32[this.loc1] += this.rangeDec.decodeDirectBits(numDirectBits - LZMA_1.LZMA.kNumAlignBits) << LZMA_1.LZMA.kNumAlignBits;
                        MEMORY_1.MEMORY.u32[this.loc1] += this.alignDecoder.reverseDecode(this.rangeDec);
                    }
                    return MEMORY_1.MEMORY.u32[this.loc1];
                };
                LzmaDecoder.prototype.initDist = function () {
                    for (var i = 0; i < LZMA_1.LZMA.kNumLenToPosStates; i++) {
                        this.posSlotDecoder[i].init();
                    }
                    this.alignDecoder.init();
                    LZMA_1.LZMA.INIT_PROBS(this.posDecoders);
                };
                LzmaDecoder.prototype.decodeProperties = function (properties) {
                    var prop = new Uint8Array(4);
                    prop[0] = properties[0];
                    if (prop[0] >= (9 * 5 * 5)) {
                        throw "Incorrect LZMA properties";
                    }
                    prop[1] = prop[0] % 9;
                    prop[0] /= 9;
                    prop[2] = prop[0] / 5;
                    prop[3] = prop[0] % 5;
                    this.lc = prop[1];
                    this.pb = prop[2];
                    this.lp = prop[3];
                    this.dictSizeInProperties = 0;
                    for (var i = 0; i < 4; i++) {
                        this.dictSizeInProperties |= properties[i + 1] << (8 * i);
                    }
                    this.dictSize = this.dictSizeInProperties;
                    if (this.dictSize < LZMA_1.LZMA.LZMA_DIC_MIN) {
                        this.dictSize = LZMA_1.LZMA.LZMA_DIC_MIN;
                    }
                };
                LzmaDecoder.prototype.updateState_Literal = function (state) {
                    if (state < 4)
                        return 0;
                    else if (state < 10)
                        return state - 3;
                    else
                        return state - 6;
                };
                LzmaDecoder.prototype.updateState_ShortRep = function (state) {
                    return state < 7 ? 9 : 11;
                };
                LzmaDecoder.prototype.updateState_Rep = function (state) {
                    return state < 7 ? 8 : 11;
                };
                LzmaDecoder.prototype.updateState_Match = function (state) {
                    return state < 7 ? 7 : 10;
                };
                LzmaDecoder.prototype.decode = function (unpackSizeDefined, unpackSize) {
                    this.init();
                    this.rangeDec.init();
                    if (unpackSizeDefined) {
                        this.outWindow.outStream = new Uint8Array(new ArrayBuffer(unpackSize));
                    }
                    var rep0 = 0, rep1 = 0, rep2 = 0, rep3 = 0;
                    var state = 0;
                    for (;;) {
                        if (unpackSizeDefined && unpackSize == 0 && !this.markerIsMandatory) {
                            if (this.rangeDec.isFinishedOK()) {
                                return LZMA_1.LZMA.LZMA_RES_FINISHED_WITHOUT_MARKER;
                            }
                        }
                        var posState = this.outWindow.totalPos & ((1 << this.pb) - 1);
                        if (this.rangeDec.decodeBit(this.isMatch, (state << LZMA_1.LZMA.kNumPosBitsMax) + posState) == 0) {
                            if (unpackSizeDefined && unpackSize == 0) {
                                return LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                            this.decodeLiteral(state, rep0);
                            state = this.updateState_Literal(state);
                            unpackSize--;
                            continue;
                        }
                        var len;
                        if (this.rangeDec.decodeBit(this.isRep, state) != 0) {
                            if (unpackSizeDefined && unpackSize == 0) {
                                return LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                            if (this.outWindow.isEmpty()) {
                                return LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                            if (this.rangeDec.decodeBit(this.isRepG0, state) == 0) {
                                if (this.rangeDec.decodeBit(this.isRep0Long, (state << LZMA_1.LZMA.kNumPosBitsMax) + posState) == 0) {
                                    state = this.updateState_ShortRep(state);
                                    this.outWindow.putByte(this.outWindow.getByte(rep0 + 1));
                                    unpackSize--;
                                    continue;
                                }
                            }
                            else {
                                var dist;
                                if (this.rangeDec.decodeBit(this.isRepG1, state) == 0) {
                                    dist = rep1;
                                }
                                else {
                                    if (this.rangeDec.decodeBit(this.isRepG2, state) == 0) {
                                        dist = rep2;
                                    }
                                    else {
                                        dist = rep3;
                                        rep3 = rep2;
                                    }
                                    rep2 = rep1;
                                }
                                rep1 = rep0;
                                rep0 = dist;
                            }
                            len = this.repLenDecoder.decode(this.rangeDec, posState);
                            state = this.updateState_Rep(state);
                        }
                        else {
                            rep3 = rep2;
                            rep2 = rep1;
                            rep1 = rep0;
                            len = this.lenDecoder.decode(this.rangeDec, posState);
                            state = this.updateState_Match(state);
                            rep0 = this.decodeDistance(len);
                            if (rep0 == 0xFFFFFFFF) {
                                return this.rangeDec.isFinishedOK() ?
                                    LZMA_1.LZMA.LZMA_RES_FINISHED_WITH_MARKER :
                                    LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                            if (unpackSizeDefined && unpackSize == 0) {
                                return LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                            if (rep0 >= this.dictSize || !this.outWindow.checkDistance(rep0)) {
                                return LZMA_1.LZMA.LZMA_RES_ERROR;
                            }
                        }
                        len += LZMA_1.LZMA.kMatchMinLen;
                        var isError = false;
                        if (unpackSizeDefined && unpackSize < len) {
                            len = unpackSize;
                            isError = true;
                        }
                        this.outWindow.copyMatch(rep0 + 1, len);
                        unpackSize -= len;
                        if (isError) {
                            return LZMA_1.LZMA.LZMA_RES_ERROR;
                        }
                    }
                };
                return LzmaDecoder;
            }());
            exports_1("LzmaDecoder", LzmaDecoder);
        }
    }
});
//# sourceMappingURL=LzmaDecoder.js.map