System.register(["./Huffman"], function(exports_1, context_1) {
    "use strict";
    var __moduleName = context_1 && context_1.id;
    var Huffman_1;
    var RawInflate;
    return {
        setters:[
            function (Huffman_1_1) {
                Huffman_1 = Huffman_1_1;
            }],
        execute: function() {
            RawInflate = (function () {
                function RawInflate(input, opt_params) {
                    this.ZLIB_RAW_INFLATE_BUFFER_SIZE = 0x8000;
                    if (!RawInflate.FixedLiteralLengthTable) {
                        var lengths = new Uint8Array(288);
                        var i, il;
                        for (i = 0, il = lengths.length; i < il; ++i) {
                            lengths[i] =
                                (i <= 143) ? 8 :
                                    (i <= 255) ? 9 :
                                        (i <= 279) ? 7 :
                                            8;
                        }
                        RawInflate.FixedLiteralLengthTable = Huffman_1.Huffman.buildHuffmanTable(lengths);
                    }
                    if (!RawInflate.FixedDistanceTable) {
                        var lengths = new Uint8Array(30);
                        var i, il;
                        for (i = 0, il = lengths.length; i < il; ++i) {
                            lengths[i] = 5;
                        }
                        RawInflate.FixedDistanceTable = Huffman_1.Huffman.buildHuffmanTable(lengths);
                    }
                    this.blocks = [];
                    this.bufferSize = this.ZLIB_RAW_INFLATE_BUFFER_SIZE;
                    this.totalpos = 0;
                    this.ip = 0;
                    this.bitsbuf = 0;
                    this.bitsbuflen = 0;
                    this.input = input;
                    this.output;
                    this.bfinal = false;
                    this.bufferType = RawInflate.BufferType.ADAPTIVE;
                    this.resize = false;
                    if (opt_params || !(opt_params = {})) {
                        if (opt_params['index']) {
                            this.ip = opt_params['index'];
                        }
                        if (opt_params['bufferSize']) {
                            this.bufferSize = opt_params['bufferSize'];
                        }
                        if (opt_params['bufferType']) {
                            this.bufferType = opt_params['bufferType'];
                        }
                        if (opt_params['resize']) {
                            this.resize = opt_params['resize'];
                        }
                    }
                    switch (this.bufferType) {
                        case RawInflate.BufferType.BLOCK:
                            this.op = RawInflate.MaxBackwardLength;
                            this.output =
                                new Uint8Array(RawInflate.MaxBackwardLength +
                                    this.bufferSize +
                                    RawInflate.MaxCopyLength);
                            break;
                        case RawInflate.BufferType.ADAPTIVE:
                            this.op = 0;
                            this.output = new Uint8Array(this.bufferSize);
                            this.expandBuffer = this.expandBufferAdaptive;
                            this.concatBuffer = this.concatBufferDynamic;
                            this.decodeHuffman = this.decodeHuffmanAdaptive;
                            break;
                        default:
                            throw new Error('invalid inflate mode');
                    }
                }
                RawInflate.prototype.decompress = function () {
                    while (!this.bfinal) {
                        this.parseBlock();
                    }
                    return this.concatBuffer();
                };
                RawInflate.prototype.parseBlock = function () {
                    var hdr = this.readBits(3);
                    if (hdr & 0x1) {
                        this.bfinal = true;
                    }
                    hdr >>>= 1;
                    switch (hdr) {
                        case 0:
                            this.parseUncompressedBlock();
                            break;
                        case 1:
                            this.parseFixedHuffmanBlock();
                            break;
                        case 2:
                            this.parseDynamicHuffmanBlock();
                            break;
                        default:
                            throw new Error('unknown BTYPE: ' + hdr);
                    }
                };
                RawInflate.prototype.readBits = function (length) {
                    var bitsbuf = this.bitsbuf;
                    var bitsbuflen = this.bitsbuflen;
                    var input = this.input;
                    var ip = this.ip;
                    var inputLength = input.length;
                    var octet;
                    while (bitsbuflen < length) {
                        if (ip >= inputLength) {
                            throw new Error('input buffer is broken');
                        }
                        bitsbuf |= input[ip++] << bitsbuflen;
                        bitsbuflen += 8;
                    }
                    octet = bitsbuf & ((1 << length) - 1);
                    bitsbuf >>>= length;
                    bitsbuflen -= length;
                    this.bitsbuf = bitsbuf;
                    this.bitsbuflen = bitsbuflen;
                    this.ip = ip;
                    return octet;
                };
                RawInflate.prototype.readCodeByTable = function (table) {
                    var bitsbuf = this.bitsbuf;
                    var bitsbuflen = this.bitsbuflen;
                    var input = this.input;
                    var ip = this.ip;
                    var inputLength = input.length;
                    var codeTable = table[0];
                    var maxCodeLength = table[1];
                    var codeWithLength;
                    var codeLength;
                    while (bitsbuflen < maxCodeLength) {
                        if (ip >= inputLength) {
                            break;
                        }
                        bitsbuf |= input[ip++] << bitsbuflen;
                        bitsbuflen += 8;
                    }
                    codeWithLength = codeTable[bitsbuf & ((1 << maxCodeLength) - 1)];
                    codeLength = codeWithLength >>> 16;
                    this.bitsbuf = bitsbuf >> codeLength;
                    this.bitsbuflen = bitsbuflen - codeLength;
                    this.ip = ip;
                    return codeWithLength & 0xffff;
                };
                RawInflate.prototype.parseUncompressedBlock = function () {
                    var input = this.input;
                    var ip = this.ip;
                    var output = this.output;
                    var op = this.op;
                    var inputLength = input.length;
                    var len;
                    var nlen;
                    var olength = output.length;
                    var preCopy;
                    this.bitsbuf = 0;
                    this.bitsbuflen = 0;
                    if (ip + 1 >= inputLength) {
                        throw new Error('invalid uncompressed block header: LEN');
                    }
                    len = input[ip++] | (input[ip++] << 8);
                    if (ip + 1 >= inputLength) {
                        throw new Error('invalid uncompressed block header: NLEN');
                    }
                    nlen = input[ip++] | (input[ip++] << 8);
                    if (len === ~nlen) {
                        throw new Error('invalid uncompressed block header: length verify');
                    }
                    if (ip + len > input.length) {
                        throw new Error('input buffer is broken');
                    }
                    switch (this.bufferType) {
                        case RawInflate.BufferType.BLOCK:
                            while (op + len > output.length) {
                                preCopy = olength - op;
                                len -= preCopy;
                                output.set(input.subarray(ip, ip + preCopy), op);
                                op += preCopy;
                                ip += preCopy;
                                this.op = op;
                                output = this.expandBuffer();
                                op = this.op;
                            }
                            break;
                        case RawInflate.BufferType.ADAPTIVE:
                            while (op + len > output.length) {
                                output = this.expandBuffer({ fixRatio: 2 });
                            }
                            break;
                        default:
                            throw new Error('invalid inflate mode');
                    }
                    output.set(input.subarray(ip, ip + len), op);
                    op += len;
                    ip += len;
                    this.ip = ip;
                    this.op = op;
                    this.output = output;
                };
                RawInflate.prototype.parseFixedHuffmanBlock = function () {
                    this.decodeHuffman(RawInflate.FixedLiteralLengthTable, RawInflate.FixedDistanceTable);
                };
                RawInflate.prototype.parseDynamicHuffmanBlock = function () {
                    var hlit = this.readBits(5) + 257;
                    var hdist = this.readBits(5) + 1;
                    var hclen = this.readBits(4) + 4;
                    var codeLengths = new Uint8Array(RawInflate.Order.length);
                    var codeLengthsTable;
                    var litlenLengths;
                    var distLengths;
                    var i;
                    for (i = 0; i < hclen; ++i) {
                        codeLengths[RawInflate.Order[i]] = this.readBits(3);
                    }
                    codeLengthsTable = Huffman_1.Huffman.buildHuffmanTable(codeLengths);
                    litlenLengths = new Uint8Array(hlit);
                    distLengths = new Uint8Array(hdist);
                    this.prev = 0;
                    this.decodeHuffman(Huffman_1.Huffman.buildHuffmanTable(this.decode.call(this, hlit, codeLengthsTable, litlenLengths)), Huffman_1.Huffman.buildHuffmanTable(this.decode.call(this, hdist, codeLengthsTable, distLengths)));
                };
                RawInflate.prototype.decode = function (num, table, lengths) {
                    var code;
                    var prev = this.prev;
                    var repeat;
                    var i;
                    for (i = 0; i < num;) {
                        code = this.readCodeByTable(table);
                        switch (code) {
                            case 16:
                                repeat = 3 + this.readBits(2);
                                while (repeat--) {
                                    lengths[i++] = prev;
                                }
                                break;
                            case 17:
                                repeat = 3 + this.readBits(3);
                                while (repeat--) {
                                    lengths[i++] = 0;
                                }
                                prev = 0;
                                break;
                            case 18:
                                repeat = 11 + this.readBits(7);
                                while (repeat--) {
                                    lengths[i++] = 0;
                                }
                                prev = 0;
                                break;
                            default:
                                lengths[i++] = code;
                                prev = code;
                                break;
                        }
                    }
                    this.prev = prev;
                    return lengths;
                };
                RawInflate.prototype.decodeHuffman = function (litlen, dist) {
                    var output = this.output;
                    var op = this.op;
                    this.currentLitlenTable = litlen;
                    var olength = output.length - RawInflate.MaxCopyLength;
                    var code;
                    var ti;
                    var codeDist;
                    var codeLength;
                    while ((code = this.readCodeByTable(litlen)) !== 256) {
                        if (code < 256) {
                            if (op >= olength) {
                                this.op = op;
                                output = this.expandBuffer();
                                op = this.op;
                            }
                            output[op++] = code;
                            continue;
                        }
                        ti = code - 257;
                        codeLength = RawInflate.LengthCodeTable[ti];
                        if (RawInflate.LengthExtraTable[ti] > 0) {
                            codeLength += this.readBits(RawInflate.LengthExtraTable[ti]);
                        }
                        code = this.readCodeByTable(dist);
                        codeDist = RawInflate.DistCodeTable[code];
                        if (RawInflate.DistExtraTable[code] > 0) {
                            codeDist += this.readBits(RawInflate.DistExtraTable[code]);
                        }
                        if (op >= olength) {
                            this.op = op;
                            output = this.expandBuffer();
                            op = this.op;
                        }
                        while (codeLength--) {
                            output[op] = output[(op++) - codeDist];
                        }
                    }
                    while (this.bitsbuflen >= 8) {
                        this.bitsbuflen -= 8;
                        this.ip--;
                    }
                    this.op = op;
                };
                RawInflate.prototype.decodeHuffmanAdaptive = function (litlen, dist) {
                    var output = this.output;
                    var op = this.op;
                    this.currentLitlenTable = litlen;
                    var olength = output.length;
                    var code;
                    var ti;
                    var codeDist;
                    var codeLength;
                    while ((code = this.readCodeByTable(litlen)) !== 256) {
                        if (code < 256) {
                            if (op >= olength) {
                                output = this.expandBuffer();
                                olength = output.length;
                            }
                            output[op++] = code;
                            continue;
                        }
                        ti = code - 257;
                        codeLength = RawInflate.LengthCodeTable[ti];
                        if (RawInflate.LengthExtraTable[ti] > 0) {
                            codeLength += this.readBits(RawInflate.LengthExtraTable[ti]);
                        }
                        code = this.readCodeByTable(dist);
                        codeDist = RawInflate.DistCodeTable[code];
                        if (RawInflate.DistExtraTable[code] > 0) {
                            codeDist += this.readBits(RawInflate.DistExtraTable[code]);
                        }
                        if (op + codeLength > olength) {
                            output = this.expandBuffer();
                            olength = output.length;
                        }
                        while (codeLength--) {
                            output[op] = output[(op++) - codeDist];
                        }
                    }
                    while (this.bitsbuflen >= 8) {
                        this.bitsbuflen -= 8;
                        this.ip--;
                    }
                    this.op = op;
                };
                RawInflate.prototype.expandBuffer = function (opt_param) {
                    if (opt_param === void 0) { opt_param = null; }
                    var buffer = new Uint8Array(this.op - RawInflate.MaxBackwardLength);
                    var backward = this.op - RawInflate.MaxBackwardLength;
                    var i;
                    var il;
                    var output = this.output;
                    buffer.set(output.subarray(RawInflate.MaxBackwardLength, buffer.length));
                    this.blocks.push(buffer);
                    this.totalpos += buffer.length;
                    output.set(output.subarray(backward, backward + RawInflate.MaxBackwardLength));
                    this.op = RawInflate.MaxBackwardLength;
                    return output;
                };
                RawInflate.prototype.expandBufferAdaptive = function (opt_param) {
                    var buffer;
                    var ratio = (this.input.length / this.ip + 1) | 0;
                    var maxHuffCode;
                    var newSize;
                    var maxInflateSize;
                    var input = this.input;
                    var output = this.output;
                    if (opt_param) {
                        if (typeof opt_param.fixRatio === 'number') {
                            ratio = opt_param.fixRatio;
                        }
                        if (typeof opt_param.addRatio === 'number') {
                            ratio += opt_param.addRatio;
                        }
                    }
                    if (ratio < 2) {
                        maxHuffCode =
                            (input.length - this.ip) / this.currentLitlenTable[2];
                        maxInflateSize = (maxHuffCode / 2 * 258) | 0;
                        newSize = maxInflateSize < output.length ?
                            output.length + maxInflateSize :
                            output.length << 1;
                    }
                    else {
                        newSize = output.length * ratio;
                    }
                    buffer = new Uint8Array(newSize);
                    buffer.set(output);
                    this.output = buffer;
                    return this.output;
                };
                RawInflate.prototype.concatBuffer = function () {
                    var pos = 0;
                    var limit = this.totalpos + (this.op - RawInflate.MaxBackwardLength);
                    var output = this.output;
                    var blocks = this.blocks;
                    var block;
                    var buffer = new Uint8Array(limit);
                    var i;
                    var il;
                    var j;
                    var jl;
                    if (blocks.length === 0) {
                        return this.output.subarray(RawInflate.MaxBackwardLength, this.op);
                    }
                    for (i = 0, il = blocks.length; i < il; ++i) {
                        block = blocks[i];
                        for (j = 0, jl = block.length; j < jl; ++j) {
                            buffer[pos++] = block[j];
                        }
                    }
                    for (i = RawInflate.MaxBackwardLength, il = this.op; i < il; ++i) {
                        buffer[pos++] = output[i];
                    }
                    this.blocks = [];
                    this.buffer = buffer;
                    return this.buffer;
                };
                RawInflate.prototype.concatBufferDynamic = function () {
                    var buffer;
                    var op = this.op;
                    if (this.resize) {
                        buffer = new Uint8Array(op);
                        buffer.set(this.output.subarray(0, op));
                    }
                    else {
                        buffer = this.output.subarray(0, op);
                    }
                    this.buffer = buffer;
                    return this.buffer;
                };
                RawInflate.BufferType = {
                    BLOCK: 0,
                    ADAPTIVE: 1
                };
                RawInflate.MaxBackwardLength = 32768;
                RawInflate.MaxCopyLength = 32768;
                RawInflate.Order = new Uint16Array([16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15]);
                RawInflate.LengthCodeTable = new Uint16Array([
                    0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 0x0008, 0x0009, 0x000a, 0x000b,
                    0x000d, 0x000f, 0x0011, 0x0013, 0x0017, 0x001b, 0x001f, 0x0023, 0x002b,
                    0x0033, 0x003b, 0x0043, 0x0053, 0x0063, 0x0073, 0x0083, 0x00a3, 0x00c3,
                    0x00e3, 0x0102, 0x0102, 0x0102
                ]);
                RawInflate.LengthExtraTable = new Uint8Array([
                    0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5,
                    5, 5, 0, 0, 0
                ]);
                RawInflate.DistCodeTable = new Uint16Array([
                    0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0007, 0x0009, 0x000d, 0x0011,
                    0x0019, 0x0021, 0x0031, 0x0041, 0x0061, 0x0081, 0x00c1, 0x0101, 0x0181,
                    0x0201, 0x0301, 0x0401, 0x0601, 0x0801, 0x0c01, 0x1001, 0x1801, 0x2001,
                    0x3001, 0x4001, 0x6001
                ]);
                RawInflate.DistExtraTable = new Uint8Array([
                    0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11,
                    11, 12, 12, 13, 13
                ]);
                return RawInflate;
            }());
            exports_1("RawInflate", RawInflate);
        }
    }
});
//# sourceMappingURL=RawInflate.js.map